@@ -166,7 +166,7 @@ impl From<serde_json::Error> for CliError {
     }
 }
 
-#[cfg(target_os = "linux")]
+#[cfg(all(target_os = "linux", feature = "magic"))]
 impl From<magic::MagicError> for CliError {
     fn from(err: magic::MagicError) -> CliError {
         CliError::Other(format!("magic error: {err:?}"))
