@@ -12,8 +12,9 @@ to adjust).
 If you want more robust, guaranteed schemata, use the "schema" or "stats" commands
 instead as they scan the entire file.
 
-On Linux, the command will also return the detected mime type. It uses the "libmagic"
-library to do this. See https://github.com/robo9k/rust-magic for more info.
+On Linux, `sniff` also acts as a general file type detector and returns the detected
+mime type if the file is not a CSV. It uses the "libmagic" library to do this.
+See https://github.com/robo9k/rust-magic for more info.
 
 For examples, see https://github.com/jqnatividad/qsv/blob/master/tests/test_sniff.rs.
 
@@ -22,7 +23,7 @@ Usage:
     qsv sniff --help
 
 sniff arguments:
-    <input>                  The CSV to sniff. This can be a local file, stdin 
+    <input>                  The file to sniff. This can be a local file, stdin 
                              or a URL (http and https schemes supported).
 
                              Note that when input is a URL, sniff will automatically
@@ -383,6 +384,13 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                 #[allow(unused_assignments)]
                 let mut chunk = Bytes::new(); // amortize the allocation
 
+                // on linux, we can short-circuit downloading by
+                // checking the file type from the first chunk
+                #[allow(unused_mut)]
+                let mut shortcircuit_flag = false;
+                #[allow(unused_mut)]
+                let mut firstchunk = Bytes::new();
+
                 // download chunks until we have the desired sample size
                 while let Some(item) = stream.next().await {
                     chunk = item.or(Err("Error while downloading file".to_string()))?;
@@ -390,6 +398,21 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     file.write_all(&chunk)
                         .map_err(|_| "Error while writing to file".to_string())?;
                     let chunk_len = chunk.len();
+
+                    // on linux, we can short-circuit downloading
+                    // by checking the file type from the first chunk
+                    #[cfg(all(target_os = "linux", feature = "magic"))]
+                    if downloaded == 0 && !snappy_flag && chunk_len >= util::FIRST_CHUNK_BUFFER_SIZE
+                    {
+                        let mime = util::sniff_filetype_from_buffer(&chunk)?;
+                        if !mime.starts_with("text/") && mime != "application/csv" {
+                            shortcircuit_flag = true;
+                            downloaded = chunk_len;
+                            firstchunk = chunk.clone();
+                            break;
+                        }
+                    }
+
                     downloaded = min(downloaded + chunk_len, total_size);
                     if show_progress {
                         progress.inc(chunk_len as u64);
@@ -430,7 +453,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
 
                 // keep the temporary file around so we can sniff it later
                 // we'll delete it when we're done
-                let (_file, path) = wtr_file
+                let (mut tmp_file, path) = wtr_file
                     .keep()
                     .or(Err("Cannot keep temporary file".to_string()))?;
 
@@ -442,11 +465,19 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
                     // before we can sniff it
                     wtr_file_path =
                         util::decompress_snappy_file(&file.path().to_path_buf(), tmpdir)?;
+                } else if shortcircuit_flag {
+                    // on linux, we can short-circuit downloading by checking the file type
+                    // from the first chunk. If the file is not a CSV file, we just write
+                    // the first chunk to a file and return
+                    wtr_file_path = path.display().to_string();
+                    tmp_file.write_all(&firstchunk)?;
+                    tmp_file.flush()?;
                 } else {
-                    // we downloaded a non-snappy file. Rewrite it so we only have the exact
-                    // sample size and truncate potentially incomplete lines. We streamed the
-                    // download and the downloaded file may be more than the sample size, and the
-                    // final line may be incomplete.
+                    // we downloaded a non-snappy file and it might be a CSV file.
+                    // Rewrite it so we only have the exact sample size and truncate potentially
+                    // incomplete lines. We do this coz we streamed the download and the downloaded
+                    // file may be more than the sample size, and the final line
+                    // may be incomplete.
                     wtr_file_path = path.display().to_string();
                     let mut wtr = Config::new(&Some(wtr_file_path.clone()))
                         .no_headers(false)
@@ -592,7 +623,7 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
             .or(Err("Cannot keep temporary file".to_string()))?;
 
         if !util::isutf8_file(&path)? {
-            return fail_clierror!("File is not UTF8-encoded nor a text/CSV file");
+            return fail_clierror!("stdin input is not UTF8-encoded");
         }
 
         let metadata = file
@@ -600,20 +631,8 @@ async fn get_file_to_sniff(args: &Args, tmpdir: &tempfile::TempDir) -> CliResult
             .map_err(|_| "Cannot get metadata for stdin file".to_string())?;
 
         let file_size = metadata.len() as usize;
-        let last_modified = match metadata.modified() {
-            Ok(time) => {
-                let timestamp = time
-                    .duration_since(SystemTime::UNIX_EPOCH)
-                    .unwrap()
-                    .as_secs();
-                let naive = chrono::NaiveDateTime::from_timestamp_opt(timestamp as i64, 0)
-                    .unwrap_or_default();
-                let datetime = chrono::DateTime::<chrono::Utc>::from_utc(naive, chrono::Utc);
-                // format the datetime to RFC3339
-                format!("{datetime}", datetime = datetime.format("%+"))
-            }
-            Err(_) => "N/A".to_string(),
-        };
+        // set last_modified to now in RFC3339 format
+        let last_modified = chrono::Utc::now().format("%+").to_string();
         let path_string = path
             .into_os_string()
             .into_string()
@@ -671,6 +690,8 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
     #[cfg(all(target_os = "linux", feature = "magic"))]
     let file_type = util::get_filetype(&sfile_info.file_to_sniff)?;
     #[cfg(all(target_os = "linux", feature = "magic"))]
+    // we also accept text/* files, as sniff may still be able to suss out
+    // if the file is a CSV, even if its not using a CSV extension
     if file_type != "application/csv" && !file_type.starts_with("text/") {
         cleanup_tempfile(sfile_info.tempfile_flag, tempfile_to_delete)?;
         if args.flag_json || args.flag_pretty_json {
@@ -678,6 +699,9 @@ pub async fn run(argv: &[&str]) -> CliResult<()> {
                 "errors": [{
                     "title": "sniff error",
                     "detail": format!("File is not a CSV file. Detected mime type: {file_type}"),
+                    "meta": {
+                        "detected_mime_type": file_type
+                    }
                 }]
             });
             if args.flag_pretty_json {
