@@ -28,9 +28,7 @@ to zero. The resulting frequency table will have all the antimode values.
 
 Summary statistics for dates are also computed when --infer-dates is enabled, with DateTime
 results in rfc3339 format and Date results in "yyyy-mm-dd" format in the UTC timezone.
-Date range, stddev, MAD & IQR are returned in days, not timestamp milliseconds. Date variance
-is currently not computed as the current streaming variance algorithm is not well suited to 
-unix epoch timestamp values.
+Date range, stddev, variance, MAD & IQR are returned in days, not timestamp milliseconds.
 
 Each column's data type is also inferred (NULL, Integer, String, Float, Date, DateTime and
 Boolean with --infer-boolean option).
@@ -56,7 +54,7 @@ If stats have already been computed for the input file with similar arguments an
 hasn't changed, the stats will be loaded from the cache instead of recomputing it.
 
 For examples, see the "boston311" test files in https://github.com/jqnatividad/qsv/tree/master/resources/test
-and https://github.com/jqnatividad/qsv/blob/f7f9c4297fb3dea685b5d0f631932b6b2ca4a99a/tests/test_stats.rs#L544.
+and https://github.com/jqnatividad/qsv/blob/4529d51273218347fef6aca15ac24e22b85b2ec4/tests/test_stats.rs#L608.
 
 Usage:
     qsv stats [options] [<input>]
@@ -116,15 +114,16 @@ stats options:
                               [default: date,time,due,open,close,created]
     --prefer-dmy              Parse dates in dmy format. Otherwise, use mdy format.
                               Ignored if --infer-dates is false.
-    --force                   Force recomputing stats even if the cache exists.
+    --force                   Force recomputing stats even if valid precomputed stats
+                              cache exists.
     -j, --jobs <arg>          The number of jobs to run in parallel.
                               This works only when the given CSV has an index.
                               Note that a file handle is opened for each job.
                               When not set, the number of jobs is set to the
                               number of CPUs detected.
     --stats-binout <file>     Write the stats to <file> in binary format.
-                              This is used by other qsv commands (currently `schema` and
-                              `tojsonl`) to load the stats into memory without having to
+                              This is used internally by other qsv commands (currently `schema`
+                              and `tojsonl`) to load cached stats into memory without having to
                               process/parse the CSV again.
 
 Common options:
@@ -137,10 +136,14 @@ Common options:
                            Must be a single character. (default: ,)
     --memcheck             Check if there is enough memory to load the entire
                            CSV into memory using CONSERVATIVE heuristics.
+                           This option is ignored when computing default, streaming
+                           statistics, as it is not needed.
 "#;
 
 /*
-DEVELOPER NOTE: stats is heavily optimized and makes extensive use of "unsafe" calls.
+DEVELOPER NOTE: stats is heavily optimized and makes extensive use of "unsafe" calls
+(primarily to skip unneeded bounds checking).
+
 It is a central command, that is used by `schema`/`validate`, `tojsonl` and Datapusher+.
 
 It was the primary reason I created the qsv fork as I needed to do GUARANTEED data type
@@ -151,10 +154,8 @@ It underpins the `schema` and `validate` commands - enabling the automatic creat
 a JSONschema based on a CSV's summary statistics; and use the generated JSONschema to
 quickly validate complex CSVs (NYC's 311 data) at almost 300,000 records/sec.
 
-These "unsafe" calls primarily skip unneeded bounds checking.
-
 To safeguard against undefined behavior, `stats` is the most extensively tested command,
-with ~480 tests.
+with ~484 tests.
 */
 
 use std::{
@@ -239,6 +240,7 @@ struct StatsArgs {
     record_count:         u64,
     date_generated:       String,
     compute_duration_ms:  u64,
+    qsv_version:          String,
 }
 
 static INFER_DATE_FLAGS: once_cell::sync::OnceCell<Vec<bool>> = OnceCell::new();
@@ -304,6 +306,7 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
         record_count:         0,
         date_generated:       String::new(),
         compute_duration_ms:  0,
+        qsv_version:          env!("CARGO_PKG_VERSION").to_string(),
     };
 
     // create a temporary file to store the <FILESTEM>.stats.csv file
@@ -421,7 +424,8 @@ pub fn run(argv: &[&str]) -> CliResult<()> {
                             == current_stats_args.flag_dates_whitelist
                         && existing_stats_args_json.flag_delimiter
                             == current_stats_args.flag_delimiter
-                        && existing_stats_args_json.flag_nulls == current_stats_args.flag_nulls)
+                        && existing_stats_args_json.flag_nulls == current_stats_args.flag_nulls
+                        && existing_stats_args_json.qsv_version == current_stats_args.qsv_version)
             {
                 log::info!("{path_file_stem}.stats.csv already exists and is current, skipping...",);
                 compute_stats = false;
@@ -1120,17 +1124,21 @@ impl Stats {
                         .iter()
                         .map(|c| String::from_utf8_lossy(c))
                         .join(",");
-                    mc_pieces.push(modes_list);
-                    mc_pieces.push(modes_count.to_string());
-                    mc_pieces.push(mode_occurrences.to_string());
+                    mc_pieces.extend_from_slice(&[
+                        modes_list,
+                        modes_count.to_string(),
+                        mode_occurrences.to_string(),
+                    ]);
 
                     // antimode/s
                     if mode_occurrences == 0 {
                         // all the values are unique
                         // so instead of returning everything, just say *ALL
-                        mc_pieces.push("*ALL".to_string());
-                        mc_pieces.push("0".to_string());
-                        mc_pieces.push("1".to_string());
+                        mc_pieces.extend_from_slice(&[
+                            "*ALL".to_string(),
+                            "0".to_string(),
+                            "1".to_string(),
+                        ]);
                     } else {
                         let (antimodes_result, antimodes_count, antimode_occurrences) =
                             v.antimodes();
@@ -1157,9 +1165,11 @@ impl Stats {
                             antimodes_list.push_str("...");
                         }
 
-                        mc_pieces.push(antimodes_list);
-                        mc_pieces.push(antimodes_count.to_string());
-                        mc_pieces.push(antimode_occurrences.to_string());
+                        mc_pieces.extend_from_slice(&[
+                            antimodes_list,
+                            antimodes_count.to_string(),
+                            antimode_occurrences.to_string(),
+                        ]);
                     }
                 }
             }
@@ -1239,9 +1249,11 @@ impl Stats {
             pieces.extend_from_slice(&[empty(), empty(), empty()]);
         } else if let Some(ref v) = self.online {
             if self.typ == TFloat || self.typ == TInteger {
-                pieces.push(util::round_num(v.mean(), round_places));
-                pieces.push(util::round_num(v.stddev(), round_places));
-                pieces.push(util::round_num(v.variance(), round_places));
+                pieces.extend_from_slice(&[
+                    util::round_num(v.mean(), round_places),
+                    util::round_num(v.stddev(), round_places),
+                    util::round_num(v.variance(), round_places),
+                ]);
             } else {
                 pieces.push(timestamp_ms_to_rfc3339(v.mean() as i64, typ));
                 // instead of returning stdev in seconds, let's return it in
@@ -1251,13 +1263,10 @@ impl Stats {
                     v.stddev() / MS_IN_DAY,
                     u32::max(round_places, DAY_DECIMAL_PLACES),
                 ));
-                // we don't know how to compute variance on timestamps
-                // it appears the current algorithm we use is not suited to the large timestamp
-                // values as the values we got during testing don't make sense, so
-                // leave it empty for now
-                // TODO: explore alternate algorithms for calculating variance
-                // see https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
-                pieces.push(empty());
+                pieces.push(util::round_num(
+                    v.variance() / (MS_IN_DAY * MS_IN_DAY),
+                    u32::max(round_places, DAY_DECIMAL_PLACES),
+                ));
             }
         } else {
             pieces.extend_from_slice(&[empty(), empty(), empty()]);
@@ -1367,31 +1376,31 @@ impl Stats {
                     // https://doc.rust-lang.org/reference/expressions/operator-expr.html#numeric-cast
                     // as values larger/smaller than what i64 can handle will automatically
                     // saturate to i64 max/min values.
-                    pieces.push(timestamp_ms_to_rfc3339(lof as i64, typ));
-                    pieces.push(timestamp_ms_to_rfc3339(lif as i64, typ));
-
-                    pieces.push(timestamp_ms_to_rfc3339(q1 as i64, typ));
-                    pieces.push(timestamp_ms_to_rfc3339(q2 as i64, typ)); // q2 = median
-                    pieces.push(timestamp_ms_to_rfc3339(q3 as i64, typ));
-                    // return iqr in days - there are 86,400,000 ms in a day
-                    pieces.push(util::round_num(
-                        (q3 - q1) / MS_IN_DAY,
-                        u32::max(round_places, DAY_DECIMAL_PLACES),
-                    ));
-
-                    pieces.push(timestamp_ms_to_rfc3339(uif as i64, typ));
-                    pieces.push(timestamp_ms_to_rfc3339(uof as i64, typ));
+                    pieces.extend_from_slice(&[
+                        timestamp_ms_to_rfc3339(lof as i64, typ),
+                        timestamp_ms_to_rfc3339(lif as i64, typ),
+                        timestamp_ms_to_rfc3339(q1 as i64, typ),
+                        timestamp_ms_to_rfc3339(q2 as i64, typ), // q2 = median
+                        timestamp_ms_to_rfc3339(q3 as i64, typ),
+                        // return iqr in days - there are 86,400,000 ms in a day
+                        util::round_num(
+                            (q3 - q1) / MS_IN_DAY,
+                            u32::max(round_places, DAY_DECIMAL_PLACES),
+                        ),
+                        timestamp_ms_to_rfc3339(uif as i64, typ),
+                        timestamp_ms_to_rfc3339(uof as i64, typ),
+                    ]);
                 } else {
-                    pieces.push(util::round_num(lof, round_places));
-                    pieces.push(util::round_num(lif, round_places));
-
-                    pieces.push(util::round_num(q1, round_places));
-                    pieces.push(util::round_num(q2, round_places)); // q2 = median
-                    pieces.push(util::round_num(q3, round_places));
-                    pieces.push(util::round_num(iqr, round_places));
-
-                    pieces.push(util::round_num(uif, round_places));
-                    pieces.push(util::round_num(uof, round_places));
+                    pieces.extend_from_slice(&[
+                        util::round_num(lof, round_places),
+                        util::round_num(lif, round_places),
+                        util::round_num(q1, round_places),
+                        util::round_num(q2, round_places), // q2 = median
+                        util::round_num(q3, round_places),
+                        util::round_num(iqr, round_places),
+                        util::round_num(uif, round_places),
+                        util::round_num(uof, round_places),
+                    ]);
                 }
                 pieces.push(util::round_num(skewness, round_places));
             }
