@@ -130,8 +130,8 @@ impl Config {
                 (Some(path), delim, snappy || file_extension.ends_with("sz"))
             }
         };
-        let sniff =
-            env::var("QSV_SNIFF_DELIMITER").is_ok() || env::var("QSV_SNIFF_PREAMBLE").is_ok();
+        let sniff = util::get_envvar_flag("QSV_SNIFF_DELIMITER")
+            || util::get_envvar_flag("QSV_SNIFF_PREAMBLE");
         let mut preamble = 0_u64;
         if sniff && path.is_some() {
             let sniff_path = path.as_ref().unwrap().to_str().unwrap();
@@ -171,8 +171,8 @@ impl Config {
             quoting: true,
             preamble_rows: preamble,
             trim: csv::Trim::None,
-            autoindex: env::var("QSV_AUTOINDEX").is_ok(),
-            prefer_dmy: env::var("QSV_PREFER_DMY").is_ok(),
+            autoindex: util::get_envvar_flag("QSV_AUTOINDEX"),
+            prefer_dmy: util::get_envvar_flag("QSV_PREFER_DMY"),
             comment: None,
             snappy,
         }
@@ -202,7 +202,7 @@ impl Config {
         if env::var("QSV_TOGGLE_HEADERS").unwrap_or_else(|_| "0".to_owned()) == "1" {
             yes = !yes;
         }
-        if env::var("QSV_NO_HEADERS").is_ok() {
+        if util::get_envvar_flag("QSV_NO_HEADERS") {
             self.no_headers = true;
         } else {
             self.no_headers = yes;
@@ -343,7 +343,7 @@ impl Config {
         let pidx = util::idx_path(Path::new(path_buf));
         let Ok(idxfile) = fs::File::create(pidx) else { return };
         let Ok(mut rdr) = self.reader_file() else { return };
-        let mut wtr = io::BufWriter::new(idxfile);
+        let mut wtr = io::BufWriter::with_capacity(DEFAULT_WTR_BUFFER_CAPACITY, idxfile);
         match csv_index::RandomAccessSimple::create(&mut rdr, &mut wtr) {
             Ok(_) => {
                 let Ok(_) = io::Write::flush(&mut wtr) else { return };
@@ -367,10 +367,10 @@ impl Config {
                 // passively trying to find an index, so we just log the warning...
                 let idx_file = match fs::File::open(util::idx_path(p)) {
                     Err(e) => {
-                        if self.autoindex {
+                        if self.autoindex && !self.snappy {
                             // however, if QSV_AUTOINDEX is set, we create the index automatically
                             self.autoindex_file();
-                            fs::File::open(util::idx_path(p)).unwrap()
+                            fs::File::open(util::idx_path(p))?
                         } else {
                             warn!("No index file found - {p:?}: {e}");
 
@@ -406,7 +406,7 @@ impl Config {
         let (data_modified, _) = util::file_metadata(&csv_file.metadata()?);
         let (idx_modified, _) = util::file_metadata(&idx_file.metadata()?);
         if data_modified > idx_modified {
-            if self.autoindex {
+            if self.autoindex && !self.snappy {
                 info!("index stale... autoindexing...");
                 self.autoindex_file();
             } else {
